import { useEffect, useState } from "react";

function calculateCambridgeDiabetesRiskScore(gender: "male" | "female", 
    prescribedAntiHypertensive: boolean, prescribedSteroids: boolean,
    age: number, bmi: number, relativesWithDiabetes: "none" | "parentOrSibling" | "parentAndSibling", smokeStatus: "never" | "past" | "current"): number {

        const a = -6.322;
        const B1 = (gender === "male") ? 0 : -0.879;
        const B2 = (prescribedAntiHypertensive) ? 1.222 : 0;
        const B3 = (prescribedSteroids) ? 2.191 : 0;
        const B4 = 0.063 * age;
        
        let B5 = 0;
        if (bmi >= 25 && bmi < 27.5) { B5 = 0.699; }
        else if (bmi >= 27.5 && bmi < 30) { B5 = 1.97; }
        else if (bmi >= 30) { B5 = 2.518; }

        let B6 = 0;
        if (relativesWithDiabetes === "parentOrSibling") { B6 = 0.728; }
        else if (relativesWithDiabetes === "parentAndSibling") { B6 = 0.753; }

        let B7 = 0;
        if (smokeStatus === "past") { B7 = -0.218; }
        else if (smokeStatus === "current") { B7 = 0.855; }

        const sum = a + B1 + B2 + B3 + B4 + B5 + B6 + B7;
        const score = (1 / (1 + Math.exp(-sum)));
        return score;
}

interface ICambridgeDiabetesRiskScoreInputProps { 
    gender: "male" | "female";
    prescribedAntiHypertensive: boolean;
    prescribedSteroids: boolean;
    age: string;
    bmi: number;
    relativesWithDiabetes: "none" | "parentOrSibling" | "parentAndSibling";
    smokeStatus: "never" | "past" | "current";

    setGender: (gender: "male" | "female") => void;
    setPrescribedAntiHypertensive: (prescribedAntiHypertensive: boolean) => void;
    setPrescribedSteroids: (prescribedSteroids: boolean) => void;
    setAge: (age: string) => void;
    setBMI: (bmi: number) => void;
    setRelativesWithDiabetes: (relativesWithDiabetes: "none" | "parentOrSibling" | "parentAndSibling") => void;
    setSmokeStatus: (smokeStatus: "never" | "past" | "current") => void;
}
export default function CambridgeDiabetesRiskScoreInput(props: ICambridgeDiabetesRiskScoreInputProps) {
    const { gender, setGender } = props;
    const { prescribedAntiHypertensive, setPrescribedAntiHypertensive } = props;
    const { prescribedSteroids, setPrescribedSteroids } = props;
    const { age, setAge } = props;
    const { bmi, setBMI } = props;
    const { relativesWithDiabetes, setRelativesWithDiabetes } = props;
    const { smokeStatus, setSmokeStatus } = props;
    const [score, setScore] = useState<number>(0);

    useEffect(() => {
        const iAge = parseInt(age);
        const tScore = calculateCambridgeDiabetesRiskScore(gender, prescribedAntiHypertensive, prescribedSteroids, isNaN(iAge) ? 0 : iAge, bmi, relativesWithDiabetes, smokeStatus);
        setScore(tScore);
    }, [gender, prescribedAntiHypertensive, prescribedSteroids, age, bmi, relativesWithDiabetes, smokeStatus, setScore]);

    return (
        <>
            <div className="flex flex-col">
                <div className="flex flex-row">
                    <div className="flex flex-col">
                        {/* Gender */}
                        <div className="mt-4 p-1 text-sm font-medium text-gray-700 dark:text-gray-200">Gender</div>
                        <select className="bg-gray-50 border border-gray-300 text-gray-900 text-sm rounded-lg focus:ring-blue-500 focus:border-blue-500 block w-full p-2.5 dark:bg-gray-700 dark:border-gray-600 dark:placeholder-gray-400 dark:text-white dark:focus:ring-blue-500 dark:focus:border-blue-500"
                            value={gender} onChange={(event) => { 
                                setGender(event.target.value === "male" ? "male" : "female") 
                            }}>
                            <option value="male">Male</option>
                            <option value="female">Female</option>
                        </select>

                        {/* Prescribed Anti-Hypertensive */}
                        <div className="mt-4 p-1 flex items-center">
                            <input type="checkbox" value="" className="w-4 h-4 text-blue-600 bg-gray-100 border-gray-300 rounded focus:ring-blue-500 dark:focus:ring-blue-600 dark:ring-offset-gray-800 focus:ring-2 dark:bg-gray-700 dark:border-gray-600"
                                checked={prescribedAntiHypertensive} onChange={(event) => {
                                    setPrescribedAntiHypertensive(event.target.checked);
                                }}></input>
                            <label className="ml-2 text-sm font-medium text-gray-900 dark:text-gray-300">Prescribed Anti-Hypertensive Medication</label>
                        </div>

                        {/* Prescribed Steroids */}
                        <div className="mt-4 p-1 flex items-center">
                            <input type="checkbox" value="" className="w-4 h-4 text-blue-600 bg-gray-100 border-gray-300 rounded focus:ring-blue-500 dark:focus:ring-blue-600 dark:ring-offset-gray-800 focus:ring-2 dark:bg-gray-700 dark:border-gray-600"
                                checked={prescribedSteroids} onChange={(event) => {
                                    setPrescribedSteroids(event.target.checked);
                                }}></input>
                            <label className="ml-2 text-sm font-medium text-gray-900 dark:text-gray-300">Prescribed Steroids</label>
                        </div>

                        {/* Age */}
                        <div className="mt-4 p-1 text-sm font-medium text-gray-700 dark:text-gray-200">Age</div>
                        <input type="text" className="bg-gray-50 border border-gray-300 text-gray-900 text-sm rounded-lg focus:ring-blue-500 focus:border-blue-500 block w-full p-2.5 dark:bg-gray-700 dark:border-gray-600 dark:placeholder-gray-400 dark:text-white dark:focus:ring-blue-500 dark:focus:border-blue-500"
                            value={age} onChange={(event) => { 
                                const age = parseInt(event.target.value);
                                if (!age) { setAge(""); }
                                if (age >= 0 && age <= 80) { setAge(age.toString()); }
                            }}></input>

                        {/* BMI */}
                        <div className="mt-4 p-1 text-sm font-medium text-gray-700 dark:text-gray-200">BMI (kg/m^2)</div>
                        <input type="text" className="bg-gray-50 border border-gray-300 text-gray-900 text-sm rounded-lg focus:ring-blue-500 focus:border-blue-500 block w-full p-2.5 dark:bg-gray-700 dark:border-gray-600 dark:placeholder-gray-400 dark:text-white dark:focus:ring-blue-500 dark:focus:border-blue-500"
                            value={bmi} onChange={(event) => {
                                const bmi = parseInt(event.target.value);
                                if (!bmi) { setBMI(0); }
                                if (bmi >= 0 && bmi <= 100) { setBMI(bmi); }
                            }}></input>

                        {/* Relatives with Diabetes */}
                        <div className="mt-4 p-1 text-sm font-medium text-gray-700 dark:text-gray-200">Relatives with Diabetes</div>
                        <select className="bg-gray-50 border border-gray-300 text-gray-900 text-sm rounded-lg focus:ring-blue-500 focus:border-blue-500 block w-full p-2.5 dark:bg-gray-700 dark:border-gray-600 dark:placeholder-gray-400 dark:text-white dark:focus:ring-blue-500 dark:focus:border-blue-500"
                            value={relativesWithDiabetes} onChange={(event) => {
                                setRelativesWithDiabetes(event.target.value === "parentAndSibling" ? "parentAndSibling" : event.target.value === "parentOrSibling" ? "parentOrSibling" : "none");
                            }}>
                            <option value="none">None</option>
                            <option value="parentOrSibling">Parent or Sibling</option>
                            <option value="parentAndSibling">Parent and Sibling</option>
                        </select>

                        {/* Smoking Status */}
                        <div className="mt-4 p-1 text-sm font-medium text-gray-700 dark:text-gray-200">Smoking History</div>
                        <select className="bg-gray-50 border border-gray-300 text-gray-900 text-sm rounded-lg focus:ring-blue-500 focus:border-blue-500 block w-full p-2.5 dark:bg-gray-700 dark:border-gray-600 dark:placeholder-gray-400 dark:text-white dark:focus:ring-blue-500 dark:focus:border-blue-500"
                            value={smokeStatus} onChange={(event) => {
                                setSmokeStatus(event.target.value === "current" ? "current" : event.target.value === "past" ? "past" : "never");
                            }}>
                            <option value="never">Never</option>
                            <option value="past">Past</option>
                            <option value="current">Current</option>
                        </select>

                        {/* Score */}
                        <div className="mt-4 p-1 text-sm font-medium text-gray-700 dark:text-gray-200">Score</div>
                        <div className="p-1 text-2xl font-medium text-gray-700 dark:text-gray-200">{(score * 100).toFixed(2)}%</div>
                        
                    </div>
                </div>
            </div>
        </>
    );
}
