import React, { useState, useEffect } from "react";
import { useRouter } from "next/router";
import * as r4 from "fhir/r4";
import { PlasmaPlatformClient } from "@plasmahealth/plasma-fhir-client";
import { getParamsFromFHIRResource } from "@/lib/risk-utils";
import CambridgeDiabetesRiskScoreInput from "@/components/CambridgeDiabetesRiskScore/CambridgeDiabetesRiskScoreInput";
import PatientHeader from "@/components/PatientHeader/PatientHeader";
const PLASMA_BASE_URL = process.env.NEXT_PUBLIC_PLASMA_BASE_URL ?? "";

export default function MainPage() {
    const router = useRouter();
    const state = router.query.state as string;
    const patientId = router.query.patientId as string;
    const [isLoading, setIsLoading] = useState<boolean>(false);
    const [patient, setPatient] = useState<r4.Patient | undefined>(undefined);
    const [smokingObservations, setSmokingObservations] = useState<r4.Observation[]>([]);
    const [bmiObservations, setBmiObservations] = useState<r4.Observation[]>([]);

    // Risk score variables...
    const [gender, setGender] = useState<"male" | "female">("female");
    const [prescribedAntiHypertensive, setPrescribedAntiHypertensive] = useState<boolean>(false);
    const [prescribedSteroids, setPrescribedSteroids] = useState<boolean>(false);
    const [age, setAge] = useState<string>("");
    const [bmi, setBMI] = useState<number>(0);
    const [relativesWithDiabetes, setRelativesWithDiabetes] = useState<"none" | "parentOrSibling" | "parentAndSibling">("none");
    const [smokeStatus, setSmokeStatus] = useState<"never" | "past" | "current">("never");

    useEffect(() => {
        if (!patientId) { return; }

        const plasma = new PlasmaPlatformClient(PLASMA_BASE_URL, state);

        setIsLoading(true);
        plasma.readPatient(patientId).then((patient: r4.Patient) => {
            console.log(patient);
            setPatient(patient);

            // Check patient's smoking status...
            plasma.readSmokingStatus(patientId).then((smokingStatus: r4.Observation[]) => {
                setSmokingObservations(smokingStatus);

                // Check patient's BMI...
                plasma.searchPatientResource<r4.Observation>(patientId, "Observation", { code: "39156-5" }).then((bmiObservations: r4.Observation[]) => {
                    setBmiObservations(bmiObservations);

                    // Initialize risk params...
                    const patientParams = getParamsFromFHIRResource(patient, smokingStatus, bmiObservations);
                    setGender(patientParams.gender);
                    setAge(patientParams.age.toString());
                    setSmokeStatus(patientParams.smokingStatus);
                    setBMI(patientParams.bmi);
                    setIsLoading(false);
                });
            });
        });
    }, [state, patientId, setIsLoading, setPatient, setSmokingObservations, setBmiObservations, setGender, setAge, setSmokeStatus, setBMI]);

    return (
        <div className="p-4 dark:bg-black">
            <h1 className="font-bold text-lg dark:text-gray-300">Cambridge Diabetes Risk Score</h1>
            <a className="text-blue-500" target="_blank" href="https://www.mdcalc.com/calc/4001/cambridge-diabetes-risk-score">https://www.mdcalc.com/calc/4001/cambridge-diabetes-risk-score</a>
            <hr className="my-2" />

            {/* Loading spinner */}
            {isLoading && (
                <div role="status" className="p-4">
                    <svg aria-hidden="true" className="w-8 h-8 mr-2 text-gray-200 animate-spin dark:text-gray-600 fill-blue-600" viewBox="0 0 100 101" fill="none" xmlns="http://www.w3.org/2000/svg">
                        <path d="M100 50.5908C100 78.2051 77.6142 100.591 50 100.591C22.3858 100.591 0 78.2051 0 50.5908C0 22.9766 22.3858 0.59082 50 0.59082C77.6142 0.59082 100 22.9766 100 50.5908ZM9.08144 50.5908C9.08144 73.1895 27.4013 91.5094 50 91.5094C72.5987 91.5094 90.9186 73.1895 90.9186 50.5908C90.9186 27.9921 72.5987 9.67226 50 9.67226C27.4013 9.67226 9.08144 27.9921 9.08144 50.5908Z" fill="currentColor"/>
                        <path d="M93.9676 39.0409C96.393 38.4038 97.8624 35.9116 97.0079 33.5539C95.2932 28.8227 92.871 24.3692 89.8167 20.348C85.8452 15.1192 80.8826 10.7238 75.2124 7.41289C69.5422 4.10194 63.2754 1.94025 56.7698 1.05124C51.7666 0.367541 46.6976 0.446843 41.7345 1.27873C39.2613 1.69328 37.813 4.19778 38.4501 6.62326C39.0873 9.04874 41.5694 10.4717 44.0505 10.1071C47.8511 9.54855 51.7191 9.52689 55.5402 10.0491C60.8642 10.7766 65.9928 12.5457 70.6331 15.2552C75.2735 17.9648 79.3347 21.5619 82.5849 25.841C84.9175 28.9121 86.7997 32.2913 88.1811 35.8758C89.083 38.2158 91.5421 39.6781 93.9676 39.0409Z" fill="currentFill"/>
                    </svg>
                    <span className="sr-only">Loading...</span>
                </div>
            )}

            {/* Patient Data */}
            {patient && (
                <div className="pt-4">
                    <h2 className="font-bold text-lg dark:text-gray-300">Patient Data:</h2>
                    <PatientHeader patient={patient} smokingObservations={smokingObservations} bmiObservations={bmiObservations} />
                    <hr className="my-2" />
                </div>
            )}

            {/* Risk Score */}
            <CambridgeDiabetesRiskScoreInput 
                gender={gender} setGender={setGender}
                prescribedAntiHypertensive={prescribedAntiHypertensive} setPrescribedAntiHypertensive={setPrescribedAntiHypertensive}
                prescribedSteroids={prescribedSteroids} setPrescribedSteroids={setPrescribedSteroids}
                age={age} setAge={setAge}
                bmi={bmi} setBMI={setBMI}
                relativesWithDiabetes={relativesWithDiabetes} setRelativesWithDiabetes={setRelativesWithDiabetes}
                smokeStatus={smokeStatus} setSmokeStatus={setSmokeStatus}
            />
        </div>
    );
}